'use strict';

exports.__esModule = true;
exports.format = exports.getPattern = exports.parse = exports.shortcuts = undefined;

var _escapeRegExp = require('lodash/escapeRegExp');

var _escapeRegExp2 = _interopRequireDefault(_escapeRegExp);

var _memoize = require('lodash/memoize');

var _memoize2 = _interopRequireDefault(_memoize);

var _routePeg = require('./routePeg');

var _routePeg2 = _interopRequireDefault(_routePeg);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var shortcuts = exports.shortcuts = {
  slug: '([a-z0-9\-\_]+)', // slug-format_1
  politeSlug: '([a-zA-Z0-9\-\_]+)', // Polite-_Form2
  strictSlug: '([a-z\-]+)', // strict-format
  username: '(\w{1,25})', // Cool_Kat43
  number: '([0-9]+)', // Any integer
  uid: '([0-9]{16,})', // Big integers
  alpha: '([a-zA-Z]+)', // AdELkf
  alnum: '([a-zA-Z0-9]+)', // alDKElff3812
  hashtag: '((%23|#)([^\s]+))', // %23GoForIt
  mention: '((%40|@)([^\s]+))', // %40ItIsMe
  word: '([a-zA-Z0-9+_]+)', // Az_09
  uuid: '([0-9A-F]{8}-' + '[0-9A-F]{4}-' + '[0-9A-F]{4}-' + '[89AB][0-9A-F]{3}-' + '[0-9A-F]{12})',
  hex: '([0-9a-fA-F]+)' // 7f14be654cfc...
};
shortcuts.id = shortcuts.number;
shortcuts.num = shortcuts.number;
shortcuts.user = shortcuts['politeSlug'];
shortcuts.username = shortcuts['politeSlug'];

var _getRegex = function _getRegex(regex) {
  regex = regex ? regex : '([^/]+)';
  var shortcutName = regex.slice(1, -1);

  if (shortcuts.hasOwnProperty(shortcutName)) {
    regex = shortcuts[shortcutName];
  }

  return regex;
};

var _getParam = function _getParam(param) {
  var rules = [];

  if (param.name.name) {
    rules.push(['', param.name.name, param.name.regex]);
  } else {
    rules.push([param.name, null, _getRegex(param.name)]);
  }

  rules.push(['=', null, (0, _escapeRegExp2.default)('=')]);

  if (param.value.name) {
    rules.push(['', param.value.name, param.value.regex]);
  } else {
    rules.push([param.value, null, _getRegex(param.value)]);
  }

  return rules;
};

var parse = exports.parse = (0, _memoize2.default)(function (path) {
  var peg = _routePeg2.default.parse(path);
  var rules = [];

  for (var i = 0; i < peg.path.length; i++) {
    var _path = peg.path[i];
    var literal = '';
    var fieldName = void 0,
        regex = void 0;

    if (_path.name !== void 0) {
      fieldName = _path.name;
      regex = _getRegex(_path.regex);
    } else {
      literal = _path;
      fieldName = null;
      regex = '(' + (0, _escapeRegExp2.default)(_path) + ')';
    }

    rules.push([literal, fieldName, regex]);
  }

  if (peg.query.length) {
    for (var x in peg.query) {
      var param = peg.query[x];
      var sym = x === '0' ? '\?' : '\&';
      rules.push.apply(rules, [[sym, null, sym]].concat(_getParam(param)));
    }
  }

  return rules;
});

/** Gets the regex rules from route path strings
  * @param {String} path
  * @returns {Object} {match: ..., groups: ...}
  */
var getPattern = exports.getPattern = (0, _memoize2.default)(function (path) {
  var rules = parse(path);
  var pattern = [];
  var groups = [];

  for (var i = 0; i < rules.length; i++) {
    var rule = rules[i];
    var lit = rule[0],
        fieldName = rule[1],
        regex = rule[2];


    pattern.push(regex);
    groups.push([regex, fieldName]);
  }

  var result = {
    match: new RegExp('^' + pattern.join('') + '(?:\\?|(?:\\/(?:\\?|$))|$)'),
    groups: groups
  };

  return result;
});

var _validOptData = function _validOptData(paramRegex, data) {
  var escapedData = (0, _escapeRegExp2.default)(data);
  return !paramRegex || paramRegex && new RegExp(paramRegex, 'g').test(escapedData);
};

var format = exports.format = function format(path) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var assertValid = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  var url = '';
  var parsedPath = parse(path);

  for (var _iterator = parsedPath, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref2;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref2 = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref2 = _i.value;
    }

    var _ref = _ref2;
    var lit = _ref[0];
    var paramName = _ref[1];
    var paramRegex = _ref[2];

    url += lit || '';
    if (paramName) {
      if (data[paramName] === null || data[paramName] === void 0) {
        throw new Error('Missing parameter \'' + paramName + '\' for ' + path);
      }

      var optVal = String(data[paramName]);
      if (assertValid || _validOptData(paramRegex, optVal)) {
        url += optVal;
      }
    }
  }

  return url;
};