import React from 'react'
import shallowCompare from 'react-addons-shallow-compare'
import shallowEqual from 'shallowequal'
import trimStart from 'lodash/trimStart'
import {url} from 'browser-essentials'
import * as context from './context'
import urlFor from './urlFor'
import * as routeParser from './routeParser'
import getPathForRouteNamed from './getPathForRouteNamed'


const isActive = (href, location) => {
  return url.resolve(location.href, href) === location.href
}


const extendHref = (path, opts) => {
  path = url.parse(path)

  if (opts.query) {
    path = path.filter(opts.query)
  }

  if (opts.hash && opts.hash.length) {
    path.hash = `#${trimStart(opts.hash, '#')}`
  }

  return url.format(path)
}


const extractHref = (context, props) => {
  const hrefObj = props.for || props.to
  let wasString = typeof hrefObj === 'string'
  let href = hrefObj
  let hrefKey = null

  if (!wasString) {
    if (hrefObj.displayName === void 0) {
      // Not a react object
      hrefKey = Object.keys(hrefObj)[0]
      href = hrefKey
    } else {
      href = hrefObj.displayName
      wasString = true
    }
  }

  if (props.for) {
    href = urlFor(context.router.routes,
                  href,
                  props,
                  props.assertValidFormat)
  }

  if (!wasString) {
    href = extendHref(href, hrefObj[hrefKey])
  }

  return href
}


export default class Link extends React.Component {
  static displayName = 'Link'
  static contextTypes = context.contextShape

  static defaultProps = {
    to: null,
    for: null,
    activeClassName: 'active',
    assertValidFormat: false
  }

  state = {
    href: null,
    isActive: false
  }

  _routerListener = null

  shouldComponentUpdate (nextProps, nextState) {
    return shallowCompare(this, nextProps, nextState)
  }

  componentWillMount () {
    this._routerListener = this.context.router.onUpdate(location => {
      const active = isActive(this.state.href, location)

      if (active !== this.state.isActive) {
        this.setState({isActive: active})
      }
    })

    this.updateIfActiveChange(this.props, true)
  }

  componentWillUnmount () {
    if (this._routerListener !== void 0) {
      this._routerListener.off()
    }
  }

  componentWillReceiveProps (nextProps) {
    this.updateIfActiveChange(nextProps)
  }

  updateIfActiveChange (nextProps, forceUpdate) {
    if (!shallowEqual(this.props, nextProps) || forceUpdate) {
      const href = extractHref(this.context, nextProps)
      const active = isActive(href, this.context.router.location)

      if (active !== this.state.isActive || href !== this.state.href) {
        this.setState({isActive: active, href})
      }
    }
  }

  click () {
    this.context.router.goto(this.state.href)
  }

  get className () {
    const cname = []

    if (this.props.className) {
      cname.push(this.props.className)
    }

    if (this.state.href && this.props.activeClassName && this.state.isActive) {
      cname.push(this.props.activeClassName)
    }

    return cname.join(' ') || null
  }

  get renderProps () {
    const props = Object.assign({}, this.props)
    props.href = this.state.href
    props.className = this.className

    // Remove default props
    for (let prop in Link.defaultProps) {
      delete props[prop]
    }
    
    // Remove url keys
    if (this.props.for !== null) {
      let forPath;

      if (typeof this.props.for !== 'string') {
        if (this.props.for.displayName !== void 0) {
          forPath = this.props.for.displayName
        } else {
          forPath = Object.keys(this.props.for)[0]
        }
      } else {
        forPath = this.props.for
      }

      forPath = getPathForRouteNamed(this.context.router.routes, forPath)
      let parsedPath = []

      try {
        parsedPath = routeParser.parse(forPath)
      } catch (e) {}

      for (let [lit, prop, rule] of parsedPath) {
        delete props[prop]
      }
    }

    return props
  }

  render () {
    return <a {...this.renderProps}/>
  }
}
