import React from 'react'
import {dom, url} from 'browser-essentials'
import {setDefinedOpt} from 'opt-setter'
import * as routeMatcher from './routeMatcher'
import Html5History from './Html5History'
import RouterContext from './RouterContext'


export default class Router extends React.Component {
  static displayName = 'Router'

  static defaultProps = {
    history: new Html5History(),
    initialLocation: null,
    maxClickSearchDepth: 10
  }

  state = {
    location: null,
    route: {path: null, params: {}, view: null},
    routes: new Map()
  }

  _listeners = []

  componentWillMount () {
    const location = this.props.initialLocation || this.props.history.location
    const routes = routeMatcher.findRoutes(this.props.children)
    const route = this.getRoute(routes, location)

    this.setState({location, routes, route})
  }

  componentDidMount () {
    const gotoListener = this.handleGoto.bind(this)
    this.props.history.on('navigate', gotoListener)

    const clickListener = this.handleClick.bind(this)
    document.addEventListener('click', clickListener)

    this._listeners.push(gotoListener, clickListener)
  }

  componentWillUnmount () {
    const [gotoListener, clickListener] = this._listeners

    this.props.history.dispose()
    document.removeEventListener('click', clickListener)
  }

  componentDidUpdate () {
    this.props.history.emit('routerUpdated', this.props.history.location)
  }

  handleGoto (location) {
    const route = this.getRoute(this.state.routes, location)
    this.setState({location, route})
  }

  handleClick (e) {
    // Ignores middle, ctrl, cmd and shift clicks
    if (dom.isMetaClick(e)) {
      return
    }

    // Only targets A tags
    const target = (e.target.nodeName === 'A') ?
                   e.target :
                   dom.findParentNode(e.target, 'A',
                                      this.props.maxClickSearchDepth)

    if (target === void 0)
      return

    // Ignores links with a `ignore` or `data-ignore` attribute
    if(
      dom.hasAttr(target, 'ignore') ||
      target.hasAttribute('target') ||
      dom.hasAttr(target, 'href') === false
    ) {
      return
    }

    // Only continues if there is an `href` or `data-href` attribute present
    e.preventDefault()

    const href = dom.getAttr(target, 'href') || dom.getAttr(target, 'url')
    let location = url.resolve(this.state.location.href, href)

    // Tells the history handler to go to the new location
    this.props.history.goto(location)
  }

  getContextProps () {
    const contextProps = Object.assign({}, this.state, Router.defaultProps)

    setDefinedOpt(contextProps, this.props, false)
    contextProps.router = this
    delete contextProps.maxClickSearchDepth

    return contextProps
  }

  getRoute (routes = this.state.routes, location = this.state.location) {
    for (let [path, route] of routes) {
      const match = routeMatcher.match(path, location.path)

      if (match !== void 0) {
        const [path, params] = match
        const view = route.view
        /*
        console.log('Matched path:', path,
                    '\nMatched params:', params,
                    '\nMatched view:', view)
        */
        return {path, params, view}
      }
    }

    throw new Error (`Route not found for location: ` +
                     `${this.state.location.href}`)
  }

  _getViewPropsFrom (props) {
    const viewProps = Object.assign({}, props)

    delete viewProps.routes
    delete viewProps.location

    return viewProps
  }

  render () {
    const App = this.props.app
    const props = this.getContextProps()
    let View = null
    const viewProps = this._getViewPropsFrom(props)

    if (this.state.route.view) {
      View = this.state.route.view
    }

    return (
      <RouterContext {...props}>
        <App {...viewProps}>
          <View {...viewProps}/>
        </App>
      </RouterContext>
    )
  }
}
