import escapeRegExp from 'lodash/escapeRegExp'
import memoize from 'lodash/memoize'
import routePeg from './routePeg'


export const shortcuts = {
  slug: '([a-z0-9\-\_]+)',           // slug-format_1
  politeSlug: '([a-zA-Z0-9\-\_]+)',  // Polite-_Form2
  strictSlug: '([a-z\-]+)',          // strict-format
  username: '(\w{1,25})',            // Cool_Kat43
  number: '([0-9]+)',                // Any integer
  uid: '([0-9]{16,})',               // Big integers
  alpha: '([a-zA-Z]+)',              // AdELkf
  alnum: '([a-zA-Z0-9]+)',           // alDKElff3812
  hashtag: '((%23|#)([^\s]+))',      // %23GoForIt
  mention: '((%40|@)([^\s]+))',      // %40ItIsMe
  word: '([a-zA-Z0-9+_]+)',          // Az_09
  uuid: '([0-9A-F]{8}-' +
          '[0-9A-F]{4}-' +
          '[0-9A-F]{4}-' +
          '[89AB][0-9A-F]{3}-' +
          '[0-9A-F]{12})',
  hex: '([0-9a-fA-F]+)'              // 7f14be654cfc...
}
shortcuts.id = shortcuts.number
shortcuts.num = shortcuts.number
shortcuts.user = shortcuts['politeSlug']
shortcuts.username = shortcuts['politeSlug']


const _getRegex = regex => {
  regex = regex ? regex : '([^/]+)'
  const shortcutName = regex.slice(1, -1)

  if (shortcuts.hasOwnProperty(shortcutName)) {
    regex = shortcuts[shortcutName]
  }

  return regex
}


const _getParam = param => {
  const rules = []

  if (param.name.name) {
    rules.push(['', param.name.name, param.name.regex])
  } else {
    rules.push([param.name, null, _getRegex(param.name)])
  }

  rules.push(['=', null, escapeRegExp('=')])

  if (param.value.name) {
    rules.push(['', param.value.name, param.value.regex])
  } else {
    rules.push([param.value, null, _getRegex(param.value)])
  }

  return rules
}


export const parse = memoize(path => {
  const peg = routePeg.parse(path)
  const rules = []

  for (let i = 0; i < peg.path.length; i++) {
    const path = peg.path[i]
    let literal = ''
    let fieldName, regex

    if (path.name !== void 0) {
      fieldName = path.name
      regex = _getRegex(path.regex)
    } else {
      literal = path
      fieldName = null
      regex = `(${escapeRegExp(path)})`
    }

    rules.push([literal, fieldName, regex])
  }

  if (peg.query.length) {
    for (let x in peg.query) {
      const param = peg.query[x]
      const sym = x === '0' ? '\?' : '\&'
      rules.push([sym, null, sym], ..._getParam(param))
    }
  }

  return rules
})


/** Gets the regex rules from route path strings
  * @param {String} path
  * @returns {Object} {match: ..., groups: ...}
  */
export const getPattern = memoize(path => {
  const rules = parse(path)
  const pattern = []
  const groups = []

  for (let i = 0; i < rules.length; i++) {
    const rule = rules[i]
    let [lit, fieldName, regex] = rule

    pattern.push(regex)
    groups.push([regex, fieldName])
  }

  const result = {
    match: new RegExp(`^${pattern.join('')}(?:\\?|(?:\\/(?:\\?|$))|$)`),
    groups: groups
  }

  return result
})


const _validOptData = (paramRegex, data) => {
  const escapedData = escapeRegExp(data)
  return !paramRegex || (paramRegex &&
                         new RegExp(paramRegex, 'g').test(escapedData))
}


export const format = (path, data = {}, assertValid = false) => {
  let url = ''
  const parsedPath = parse(path)

  for (let [lit, paramName, paramRegex] of parsedPath) {
    url += lit || ''
    if (paramName) {
      if (data[paramName] === null || data[paramName] === void 0) {
        throw new Error(`Missing parameter '${paramName}' for ${path}`)
      }

      const optVal = String(data[paramName])
      if (assertValid || _validOptData(paramRegex, optVal)) {
        url += optVal
      }
    }
  }

  return url
}
